@@ -3,9 +3,12 @@ Provides a simple Daemon class to ease the process of forking a
 python application on Unix systems.
 """
 
-VERSION = (1,0,1)
+VERSION = (1, 3, 0)
 
-import ConfigParser
+try:
+    import configparser
+except ImportError:
+    import ConfigParser as configparser
 import errno
 import grp
 import logging
@@ -20,8 +23,19 @@ import time
 class Daemon(object):
     """Daemon base class"""
 
-    default_conf = ''    # override this
-    section = 'daemon'   # override this
+    def __init__(self):
+        """Override to change where the daemon looks for config information.
+
+        By default we use the 'daemon' section of a file with the same name
+        as the python module holding the subclass, ending in .conf
+        instead of .py.
+        """
+        if not hasattr(self, 'default_conf'):
+            # Grabs the filename that the Daemon subclass resides in...
+            self.daemon_file = sys.modules[self.__class__.__module__].__file__
+            self.default_conf = self.daemon_file.rpartition('.')[0] + '.conf'
+        if not hasattr(self, 'section'):
+            self.section = 'daemon'
 
     def setup_root(self):
         """Override to perform setup tasks with root privileges.
@@ -79,15 +93,23 @@ class Daemon(object):
     def read_basic_config(self):
         """Read basic options from the daemon config file"""
         self.config_filename = self.options.config_filename
-        cp = ConfigParser.ConfigParser()
+        cp = configparser.ConfigParser(defaults={
+            'logmaxmb': '0',
+            'logbackups': '0',
+            'loglevel': 'info',
+            'uid': str(os.getuid()),
+            'gid': str(os.getgid()),
+        })
         cp.read([self.config_filename])
         self.config_parser = cp
 
         try:
-            self.uid, self.gid = get_uid_gid(cp, self.section)
-        except ValueError, e:
+            self.uid, self.gid = map(int, get_uid_gid(cp, self.section))
+        except ValueError as e:
             sys.exit(str(e))
 
+        self.logmaxmb = int(cp.get(self.section, 'logmaxmb'))
+        self.logbackups = int(cp.get(self.section, 'logbackups'))
         self.pidfile = cp.get(self.section, 'pidfile')
         self.logfile = cp.get(self.section, 'logfile')
         self.loglevel = cp.get(self.section, 'loglevel')
@@ -161,8 +183,8 @@ class Daemon(object):
                 try:
                     # poll the process state
                     os.kill(pid, 0)
-                except OSError, why:
-                    if why[0] == errno.ESRCH:
+                except OSError as why:
+                    if why.errno == errno.ESRCH:
                         # process has died
                         break
                     else:
@@ -187,15 +209,15 @@ class Daemon(object):
         if self.gid:
             try:
                 os.setgid(self.gid)
-            except OSError, (code, message):
+            except OSError as err:
                 sys.exit("can't setgid(%d): %s, %s" %
-                (self.gid, code, message))
+                (self.gid, err.errno, err.strerror))
         if self.uid:
             try:
                 os.setuid(self.uid)
-            except OSError, (code, message):
+            except OSError as err:
                 sys.exit("can't setuid(%d): %s, %s" %
-                (self.uid, code, message))
+                (self.uid, err.errno, err.strerror))
 
     def chown(self, fn):
         """Change the ownership of a file to match the daemon uid/gid"""
@@ -208,9 +230,9 @@ class Daemon(object):
                 gid = os.stat(fn).st_gid
             try:
                 os.chown(fn, uid, gid)
-            except OSError, (code, message):
+            except OSError as err:
                 sys.exit("can't chown(%s, %d, %d): %s, %s" %
-                (repr(fn), uid, gid, code, message))
+                (repr(fn), uid, gid, err.errno, err.strerror))
 
     def start_logging(self):
         """Configure the logging module"""
@@ -221,7 +243,11 @@ class Daemon(object):
 
         handlers = []
         if self.logfile:
-            handlers.append(logging.FileHandler(self.logfile))
+            if not self.logmaxmb:
+                handlers.append(logging.FileHandler(self.logfile))
+            else:
+                from logging.handlers import RotatingFileHandler
+                handlers.append(RotatingFileHandler(self.logfile, maxBytes=self.logmaxmb * 1024 * 1024, backupCount=self.logbackups))
             self.chown(self.logfile)
         if not self.options.daemonize:
             # also log to stderr
@@ -246,19 +272,19 @@ class Daemon(object):
         # based on twisted/scripts/twistd.py
         if os.path.exists(self.pidfile):
             try:
-                pid = int(open(self.pidfile).read().strip())
+                pid = int(open(self.pidfile, 'rb').read().decode('utf-8').strip())
             except ValueError:
                 msg = 'pidfile %s contains a non-integer value' % self.pidfile
                 sys.exit(msg)
             try:
                 os.kill(pid, 0)
-            except OSError, (code, text):
-                if code == errno.ESRCH:
+            except OSError as err:
+                if err.errno == errno.ESRCH:
                     # The pid doesn't exist, so remove the stale pidfile.
                     os.remove(self.pidfile)
                 else:
                     msg = ("failed to check status of process %s "
-                           "from pidfile %s: %s" % (pid, self.pidfile, text))
+                           "from pidfile %s: %s" % (pid, self.pidfile, err.strerror))
                     sys.exit(msg)
             else:
                 msg = ('another instance seems to be running (pid %s), '
@@ -284,7 +310,7 @@ class Daemon(object):
     def write_pid(self):
         """Write to the pid file"""
         if self.pidfile:
-            open(self.pidfile, 'wb').write(str(os.getpid()))
+            open(self.pidfile, 'wb').write(str(os.getpid()).encode('utf-8'))
 
     def remove_pid(self):
         """Delete the pid file"""
@@ -300,7 +326,7 @@ def get_uid_gid(cp, section):
     uid = cp.get(section, 'uid')
     if uid:
         try:
-            int(uid)
+            uid = int(uid)
         except ValueError:
             # convert user name to uid
             try:
@@ -311,7 +337,7 @@ def get_uid_gid(cp, section):
     gid = cp.get(section, 'gid')
     if gid:
         try:
-            int(gid)
+            gid = int(gid)
         except ValueError:
             # convert group name to gid
             try:
@@ -327,16 +353,16 @@ def daemonize():
     # swiped from twisted/scripts/twistd.py
     # See http://www.erlenstar.demon.co.uk/unix/faq_toc.html#TOC16
     if os.fork():   # launch child and...
-        os._exit(0) # kill off parent
+        os._exit(0)  # kill off parent
     os.setsid()
     if os.fork():   # launch child and...
-        os._exit(0) # kill off parent again.
-    os.umask(077)
-    null=os.open('/dev/null', os.O_RDWR)
+        os._exit(0)  # kill off parent again.
+    os.umask(63)  # 077 in octal
+    null = os.open('/dev/null', os.O_RDWR)
     for i in range(3):
         try:
             os.dup2(null, i)
-        except OSError, e:
+        except OSError as e:
             if e.errno != errno.EBADF:
                 raise
     os.close(null)
